@@ -27,8 +27,8 @@
 #'@param x Object coercible to xts or matrix.
 #'@param y Object coercible to xts or matrix.
 #'@param n Number of periods to use in the window or, if
-#'\code{cumulative=TRUE}, the number of obversations to use before the first
-#'result is returned.
+#'\code{cumulative=TRUE}, the number of observations to use before the first
+#'result is returned. Must be between 1 and \code{nrow(x)}, inclusive.
 #'@param cumulative Logical, use from-inception calculation?
 #'@param sample Logical, sample covariance if \code{TRUE} (denominator of
 #'\code{n-1})
@@ -59,52 +59,40 @@
 #'@author Joshua Ulrich
 #'@keywords ts
 #'@rdname runFun
-#'@export
 "runSum" <-
 function(x, n=10, cumulative=FALSE) {
 
   x <- try.xts(x, error=as.matrix)
 
-  if( n < 1 || n > NROW(x) ) stop("Invalid 'n'")
+  if( n < 1 || n > NROW(x) )
+    stop(sprintf("n = %d is outside valid range: [1, %d]", n, NROW(x)))
 
   if(NCOL(x) > 1) {
     stop("ncol(x) > 1. runSum only supports univariate 'x'")
   }
 
-  # Count NAs, ensure they're only at beginning of data.
-  NAs <- sum(is.na(x))
-  if( NAs > 0 ) {
-    if( any( is.na(x[-(1:NAs)]) ) ) stop("Series contains non-leading NAs")
-    if( NAs + n > NROW(x) ) stop("not enough non-NA values")
-  }
-  beg <- 1 + NAs
-  len <- NROW(x) - NAs
-
-  # Initialize result vector 
-  result <- double(NROW(x))
-
   if(cumulative) {
-    result[beg:NROW(x)] <- cumsum(x[beg:NROW(x)])
-  } else {
-    result[(n+beg-1)] <- sum(x[beg:(n+beg-1)])
+    # Count NAs, ensure they're only at beginning of data.
+    NAs <- sum(is.na(x))
+    if( NAs > 0 ) {
+      if( any( is.na(x[-(1:NAs)]) ) ) stop("Series contains non-leading NAs")
+      if( NAs + n > NROW(x) ) stop("not enough non-NA values")
+    }
+    beg <- 1 + NAs
+    len <- NROW(x) - NAs
 
-    # Call Fortran routine
-    result <- .Fortran( "runsum",
-                     ia = as.double(x[beg:NROW(x)]),
-                     lia = as.integer(len),
-                     n = as.integer(n),
-                     oa = as.double(result[beg:NROW(x)]),
-                     loa = as.integer(len),
-                     PACKAGE = "TTR",
-                     DUP = TRUE )$oa
-    
-    # Prepend NAs from original data
-    result <- c( rep( NA, NAs ), result )
+    # Initialize result vector
+    result <- double(NROW(x))
+
+    result[beg:NROW(x)] <- cumsum(x[beg:NROW(x)])
+
+    # Replace 1:(n-1) with NAs
+    is.na(result) <- c(1:(n-1+NAs))
+  } else {
+    # Call C routine
+    result <- .Call("runsum", x, n, PACKAGE = "TTR")
   }
   
-  # Replace 1:(n-1) with NAs
-  is.na(result) <- c(1:(n-1+NAs))
-
   # Convert back to original class
   reclass(result, x)
 }
@@ -112,51 +100,40 @@ function(x, n=10, cumulative=FALSE) {
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "runMin" <-
 function(x, n=10, cumulative=FALSE) {
 
   x <- try.xts(x, error=as.matrix)
 
-  if( n < 1 || n > NROW(x) ) stop("Invalid 'n'")
+  if( n < 1 || n > NROW(x) )
+    stop(sprintf("n = %d is outside valid range: [1, %d]", n, NROW(x)))
 
   if(NCOL(x) > 1) {
     stop("ncol(x) > 1. runMin only supports univariate 'x'")
   }
 
-  # Count NAs, ensure they're only at beginning of data, then remove.
-  NAs <- sum( is.na(x) )
-  if( NAs > 0 ) {
-    if( any( is.na(x[-(1:NAs)]) ) ) stop("Series contains non-leading NAs")
-    if( NAs + n > NROW(x) ) stop("not enough non-NA values")
-  }
-  beg <- 1 + NAs
-  len <- NROW(x) - NAs
-
-  # Initialize result vector 
-  result <- double(NROW(x))
-  
   if(cumulative) {
+    # Count NAs, ensure they're only at beginning of data, then remove.
+    NAs <- sum( is.na(x) )
+    if( NAs > 0 ) {
+      if( any( is.na(x[-(1:NAs)]) ) ) stop("Series contains non-leading NAs")
+      if( NAs + n > NROW(x) ) stop("not enough non-NA values")
+    }
+    beg <- 1 + NAs
+    len <- NROW(x) - NAs
+
+    # Initialize result vector
+    result <- double(NROW(x))
+
     result[beg:NROW(x)] <- cummin(x[beg:NROW(x)])
+
+    # Replace 1:(n-1) with NAs
+    is.na(result) <- c(1:(n-1+NAs))
   } else {
-    result[(n+beg-1)] <- min(x[beg:(n+beg-1)])
-
-    result <- .Fortran( "runmin",
-                     ia = as.double(x[beg:NROW(x)]),
-                     lia = as.integer(len),
-                     n = as.integer(n),
-                     oa = as.double(result[beg:NROW(x)]),
-                     loa = as.integer(len),
-                     PACKAGE = "TTR",
-                     DUP = TRUE )$oa
-
-    # Prepend NAs from original data
-    result <- c( rep( NA, NAs ), result )
+    # Call C routine
+    result <- .Call("runmin", x, n, PACKAGE = "TTR")
   }
   
-  # Replace 1:(n-1) with NAs
-  is.na(result) <- c(1:(n-1+NAs))
-
   # Convert back to original class
   reclass(result, x)
 }
@@ -164,49 +141,44 @@ function(x, n=10, cumulative=FALSE) {
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "runMax" <-
 function(x, n=10, cumulative=FALSE) {
 
   x <- try.xts(x, error=as.matrix)
   
-  if( n < 1 || n > NROW(x) ) stop("Invalid 'n'")
-
-  # Count NAs, ensure they're only at beginning of data, then remove.
-  NAs <- sum( is.na(x) )
-  if( NAs > 0 ) {
-    if( any( is.na(x[-(1:NAs)]) ) ) stop("Series contains non-leading NAs")
-    if( NAs + n > NROW(x) ) stop("not enough non-NA values")
-  }
-  beg <- 1 + NAs
-  len <- NROW(x) - NAs
+  if( n < 1 || n > NROW(x) )
+    stop(sprintf("n = %d is outside valid range: [1, %d]", n, NROW(x)))
 
   if(NCOL(x) > 1) {
     stop("ncol(x) > 1. runMax only supports univariate 'x'")
   }
 
-  # Initialize result vector 
-  result <- double(NROW(x))
-
   if(cumulative) {
+    # Count NAs, ensure they're only at beginning of data, then remove.
+    NAs <- sum( is.na(x) )
+    if( NAs > 0 ) {
+      if( any( is.na(x[-(1:NAs)]) ) ) stop("Series contains non-leading NAs")
+      if( NAs + n > NROW(x) ) stop("not enough non-NA values")
+    }
+    beg <- 1 + NAs
+    len <- NROW(x) - NAs
+
+    if(NCOL(x) > 1) {
+      stop("ncol(x) > 1. runMax only supports univariate 'x'")
+    }
+
+    # Initialize result vector
+    result <- double(NROW(x))
+
     result[beg:NROW(x)] <- cummax(x[beg:NROW(x)])
+
+    # Replace 1:(n-1) with NAs and prepend NAs from original data
+    is.na(result) <- c(1:(n-1+NAs))
   } else {
-    result[(n+beg-1)] <- max(x[beg:(n+beg-1)])
-
-    result <- .Fortran( "runmax",
-                     ia = as.double(x[beg:NROW(x)]),
-                     lia = as.integer(len),
-                     n = as.integer(n),
-                     oa = as.double(result[beg:NROW(x)]),
-                     loa = as.integer(len),
-                     PACKAGE = "TTR",
-                     DUP = TRUE )$oa
+    # Call C routine
+    result <- .Call("runmax", x, n, PACKAGE = "TTR")
   }
 
-  # Replace 1:(n-1) with NAs and prepend NAs from original data
-  is.na(result) <- c(1:(n-1))
-  result <- c( rep( NA, NAs ), result )
-
   # Convert back to original class
   reclass(result, x)
 }
@@ -214,7 +186,6 @@ function(x, n=10, cumulative=FALSE) {
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "runMean" <-
 function(x, n=10, cumulative=FALSE) {
 
@@ -230,22 +201,13 @@ function(x, n=10, cumulative=FALSE) {
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "runMedian" <-
 function(x, n=10, non.unique="mean", cumulative=FALSE) {
 
   x <- try.xts(x, error=as.matrix)
 
-  if( n < 1 || n > NROW(x) ) stop("Invalid 'n'")
-
-  # Count NAs, ensure they're only at beginning of data, then remove.
-  NAs <- sum( is.na(x) )
-  if( NAs > 0 ) {
-    if( any( is.na(x[-(1:NAs)]) ) ) stop("Series contains non-leading NAs")
-    if( NAs + n > NROW(x) ) stop("not enough non-NA values")
-  }
-  beg <- 1 + NAs
-  len <- NROW(x) - NAs
+  if( n < 1 || n > NROW(x) )
+    stop(sprintf("n = %d is outside valid range: [1, %d]", n, NROW(x)))
 
   if(NCOL(x) > 1) {
     stop("ncol(x) > 1. runMedian only supports univariate 'x'")
@@ -253,22 +215,10 @@ function(x, n=10, non.unique="mean", cumulative=FALSE) {
 
   # Non-unique median
   non.unique <- match.arg(non.unique, c('mean','max','min'))
-  non.unique <- switch( non.unique, mean=0, max=1, min=-1 )
-  
-  # Call Fortran routine
-  result <- .Fortran( "runmedian",
-                   ia = as.double(x[beg:NROW(x)]),
-                   n = as.integer(n),
-                   oa = double(len),
-                   la = as.integer(len),
-                   ver = as.integer(non.unique),
-                   cu = as.integer(cumulative),
-                   PACKAGE = "TTR",
-                   DUP = TRUE )$oa
+  non.unique <- switch(non.unique, mean=0L, max=1L, min=-1L)
 
-  # Replace 1:(n-1) with NAs and prepend NAs from original data
-  is.na(result) <- c(1:(n-1))
-  result <- c( rep( NA, NAs ), result )
+  # Call C routine
+  result <- .Call("runmedian", x, n, non.unique, cumulative, PACKAGE = "TTR")
 
   # Convert back to original class
   reclass(result, x)
@@ -277,7 +227,6 @@ function(x, n=10, non.unique="mean", cumulative=FALSE) {
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "runCov" <-
 function(x, y, n=10, use="all.obs", sample=TRUE, cumulative=FALSE) {
 
@@ -289,7 +238,8 @@ function(x, y, n=10, use="all.obs", sample=TRUE, cumulative=FALSE) {
     xy <- cbind( as.vector(x), as.vector(y) )
   }
 
-  if( n < 1 || n > NROW(x) ) stop("Invalid 'n'")
+  if( n < 1 || n > NROW(x) )
+    stop(sprintf("n = %d is outside valid range: [1, %d]", n, NROW(x)))
 
   if(NCOL(x) > 1 || NCOL(y) > 1) {
     stop("ncol(x) > 1 or ncol(y) > 1.",
@@ -298,39 +248,8 @@ function(x, y, n=10, use="all.obs", sample=TRUE, cumulative=FALSE) {
 
   # "all.obs", "complete.obs", "pairwise.complete.obs"
 
-  # Count NAs, ensure they're only at beginning of data, then remove.
-  xNAs <- sum( is.na(x) )
-  yNAs <- sum( is.na(y) )
-  NAs <- max( xNAs, yNAs )
-  if( NAs > 0 ) {
-    if( any( is.na(xy[-(1:NAs),]) ) ) stop("Series contain non-leading NAs")
-    if( NAs + n > NROW(x) ) stop("not enough non-NA values")
-  }
-  beg <- 1 + NAs
-  len <- NROW(xy) - NAs
-  
-  xCenter <- runMean(x, n, cumulative)
-  xCenter[1:(NAs+n-1)] <- 0
-  yCenter <- runMean(y, n, cumulative)
-  yCenter[1:(NAs+n-1)] <- 0
-
-  # Call Fortran routine
-  result <- .Fortran( "runCov",
-                   rs1 = as.double(x[beg:NROW(xy)]),
-                   avg1 = as.double(xCenter[beg:NROW(xy)]),
-                   rs2 = as.double(y[beg:NROW(xy)]),
-                   avg2 = as.double(yCenter[beg:NROW(xy)]),
-                   la = as.integer(len),
-                   n = as.integer(n),
-                   samp = as.integer(sample),
-                   oa = double(len),
-                   cu = as.integer(cumulative),
-                   PACKAGE = "TTR",
-                   DUP = TRUE )$oa
-
-  # Replace 1:(n-1) with NAs and prepend NAs from original data
-  is.na(result) <- c(1:(n-1))
-  result <- c( rep( NA, NAs ), result )
+  # Call C routine
+  result <- .Call("runcov", x, y, n, sample, cumulative, PACKAGE = "TTR")
 
   # Convert back to original class
   # Should the attributes of *both* x and y be retained?
@@ -340,7 +259,6 @@ function(x, y, n=10, use="all.obs", sample=TRUE, cumulative=FALSE) {
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "runCor" <-
 function(x, y, n=10, use="all.obs", sample=TRUE, cumulative=FALSE) {
 
@@ -354,7 +272,6 @@ function(x, y, n=10, use="all.obs", sample=TRUE, cumulative=FALSE) {
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "runVar" <-
 function(x, y=NULL, n=10, sample=TRUE, cumulative=FALSE) {
 
@@ -367,7 +284,6 @@ function(x, y=NULL, n=10, sample=TRUE, cumulative=FALSE) {
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "runSD" <-
 function(x, n=10, sample=TRUE, cumulative=FALSE) {
 
@@ -380,32 +296,22 @@ function(x, n=10, sample=TRUE, cumulative=FALSE) {
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "runMAD" <-
 function(x, n=10, center=NULL, stat="median",
          constant=1.4826, non.unique="mean", cumulative=FALSE) {
 
   x <- try.xts(x, error=as.matrix)
 
-  if( n < 1 || n > NROW(x) ) stop("Invalid 'n'")
+  if( n < 1 || n > NROW(x) )
+    stop(sprintf("n = %d is outside valid range: [1, %d]", n, NROW(x)))
 
   if(NCOL(x) > 1) {
     stop("ncol(x) > 1. runMAD only supports univariate 'x'")
   }
 
-  # Count NAs, ensure they're only at beginning of data, then remove.
-  NAs <- sum( is.na(x) )
-  if( NAs > 0 ) {
-    if( any( is.na(x[-(1:NAs)]) ) ) stop("Series contains non-leading NAs")
-    if( NAs + n > NROW(x) ) stop("not enough non-NA values")
-  }
-  beg <- 1 + NAs
-  len <- NROW(x) - NAs
-  
   if(is.null(center)) {
     center <- runMedian(x, n, cumulative=cumulative)
   }
-  center[1:(NAs+n-1)] <- 0
 
   # Mean or Median absolute deviation?
   median <- match.arg(stat, c("mean","median"))
@@ -414,26 +320,13 @@ function(x, n=10, center=NULL, stat="median",
   # Non-unique median
   non.unique <- match.arg(non.unique, c('mean','max','min'))
   non.unique <- switch( non.unique, mean=0, max=1, min=-1 )
-  
-  # Call Fortran routine
-  result <- .Fortran( "runMAD",
-                   rs = as.double(x[beg:NROW(x)]),      # raw series
-                   cs = as.double(center[beg:NROW(x)]), # center series
-                   la = as.integer(len),                # length of input arrays
-                   n = as.integer(n),                   # size of rolling window
-                   oa = double(len),                    # output array
-                   stat = as.integer(median),           # center statistic
-                   ver = as.integer(non.unique),        # median type
-                   cu = as.integer(cumulative),         # from inception
-                   PACKAGE = "TTR",
-                   DUP = TRUE )$oa
+
+  # Call C routine
+  result <- .Call("runmad", x, center, n, median, non.unique, cumulative,
+                  PACKAGE = "TTR")
 
   if( median ) result <- result * constant
 
-  # Replace 1:(n-1) with NAs and prepend NAs from original data
-  is.na(result) <- c(1:(n-1))
-  result <- c( rep( NA, NAs ), result )
-
   # Convert back to original class
   reclass(result, x)
 }
@@ -441,13 +334,13 @@ function(x, n=10, center=NULL, stat="median",
 #-------------------------------------------------------------------------#
 
 #'@rdname runFun
-#'@export
 "wilderSum" <-
 function(x, n=10) {
 
   x <- try.xts(x, error=as.matrix)
 
-  if( n < 1 || n > NROW(x) ) stop("Invalid 'n'")
+  if( n < 1 || n > NROW(x) )
+    stop(sprintf("n = %d is outside valid range: [1, %d]", n, NROW(x)))
 
   if(NCOL(x) > 1) {
     stop("ncol(x) > 1. wilderSum only supports univariate 'x'")
