@@ -98,12 +98,12 @@
 #'@references The following site(s) were used to code/document these
 #'indicators:
 #'\cr Stochastic Oscillator:\cr
-#'\url{http://www.fmlabs.com/reference/StochasticOscillator.htm}\cr
+#'\url{https://www.fmlabs.com/reference/StochasticOscillator.htm}\cr
 #'\url{https://www.metastock.com/Customer/Resources/TAAZ/?p=106}\cr
 #'\url{https://www.linnsoft.com/techind/stochastics}\cr
-#'\url{http://www.stockcharts.com/school/doku.php?id=chart_school:technical_indicators:stochastic_oscillator_fast_slow_and_full}\cr
+#'\url{https://www.stockcharts.com/school/doku.php?id=chart_school:technical_indicators:stochastic_oscillator_fast_slow_and_full}\cr
 #'\cr SMI:\cr
-#'\url{http://www.fmlabs.com/reference/default.htm?url=SMI.htm}\cr
+#'\url{https://www.fmlabs.com/reference/default.htm?url=SMI.htm}\cr
 #'@keywords ts
 #'@examples
 #'
@@ -125,7 +125,6 @@
 #'
 #' stochRSI <- stoch( RSI(ttrc[,"Close"]) )
 #'@rdname stochastics
-#'@export
 "stoch" <-
 function(HLC, nFastK=14, nFastD=3, nSlowD=3, maType, bounded=TRUE, smooth=1, ...) {
 
@@ -154,7 +153,7 @@ function(HLC, nFastK=14, nFastD=3, nSlowD=3, maType, bounded=TRUE, smooth=1, ...
     lmin <- runMin( low, nFastK)
   } else {
     hmax <- runMax(c(high[1],high[-NROW(HLC)]), nFastK)
-    lmin <- runMax(c( low[1], low[-NROW(HLC)]), nFastK)
+    lmin <- runMin(c( low[1], low[-NROW(HLC)]), nFastK)
   }
 
   num <- close - lmin
@@ -220,7 +219,6 @@ function(HLC, nFastK=14, nFastD=3, nSlowD=3, maType, bounded=TRUE, smooth=1, ...
 #-------------------------------------------------------------------------#
 
 #'@rdname stochastics
-#'@export
 "SMI" <-
 function(HLC, n=13, nFast=2, nSlow=25, nSig=9, maType, bounded=TRUE, ...) {
 
