@@ -53,6 +53,12 @@ test.runMin <- function() {
   checkException( runMin(input$all$Close, n = NROW(input$all) + 1) )
   checkEqualsNumeric( tail(runMin(input$all$Close,250),1), min(input$all$Close) )
 }
+test.runMin.cumulative <- function() {
+  ttr <- runMin(input$all$Close, 1, TRUE)
+  base <- cummin(input$all$Close)
+  is.na(base) <- 1
+  checkEqualsNumeric(base, ttr)
+}
 
 # Max
 test.runMax <- function() {
@@ -66,6 +72,12 @@ test.runMax <- function() {
   checkException( runMax(input$all$Close, n = NROW(input$all) + 1) )
   checkEqualsNumeric( tail(runMax(input$all$Close,250),1), max(input$all$Close) )
 }
+test.runMax.cumulative <- function() {
+  ttr <- runMax(input$all$Close, 1, TRUE)
+  base <- cummax(input$all$Close)
+  is.na(base) <- 1
+  checkEqualsNumeric(base, ttr)
+}
 
 # Mean
 test.runMean <- function() {
@@ -79,6 +91,13 @@ test.runMean <- function() {
   checkException( runMean(input$all$Close, n = NROW(input$all) + 1) )
   checkEqualsNumeric( tail(runMean(input$all$Close,250),1), mean(input$all$Close) )
 }
+test.runMean.cumulative <- function() {
+  ttr <- runMean(input$all$Close, 5, TRUE)
+  base <- cumsum(input$all$Close) / seq_along(input$all$Close)
+  is.na(base) <- 1:4
+  checkEqualsNumeric(base, ttr)
+}
+
 
 # Median
 test.runMedian <- function() {
@@ -92,6 +111,34 @@ test.runMedian <- function() {
   checkException( runMedian(input$all$Close, n = NROW(input$all) + 1) )
   checkEqualsNumeric( tail(runMedian(input$all$Close,250),1), median(input$all$Close) )
 }
+test.runMedian.cumulative <- function() {
+  cummedian <- compiler::cmpfun(
+    function(x) {
+      med <- x * NA_real_
+      for (i in seq_along(x)) {
+        med[i] <- median(x[1:i])
+      }
+      med
+    }
+  )
+  base <- cummedian(input$all$Close)
+  is.na(base) <- 1:4
+  ttr <- runMedian(input$all$Close, 5, "mean", TRUE)
+  checkEqualsNumeric(base, ttr)
+
+  is.na(base) <- 1:5
+  ttr <- runMedian(input$all$Close, 6, "mean", TRUE)
+  checkEqualsNumeric(base, ttr)
+}
+
+test.runMedian.cumulative.leading.NA <- function() {
+  na <- rep(NA, 10)
+  x <- input$all$Close
+  xmed <- runMedian(x, 1, "mean", TRUE)
+  y <- c(na, input$all$Close)
+  ymed <- runMedian(y, 1, "mean", TRUE)
+  checkEqualsNumeric(ymed, c(na, xmed))
+}
 
 # Covariance
 test.runCov <- function() {
@@ -106,6 +153,28 @@ test.runCov <- function() {
   checkException( runCov(input$all$Close, n = NROW(input$all) + 1) )
   checkEqualsNumeric( tail(runCov(input$all$High, input$all$Low, 250),1), cov(input$all$High, input$all$Low) )
 }
+test.runCov.cumulative <- function() {
+  cumcov <- compiler::cmpfun(
+    function(x) {
+      cov <- x * NA_real_
+      for (i in seq_along(x)) {
+        y <- x[1:i]
+        cov[i] <- cov(y, y)
+      }
+      cov
+    }
+  )
+  x <- input$all$Close
+  base <- cumcov(x)
+  is.na(base) <- 1:4
+  ttr <- runCov(x, x, 5, "all.obs", TRUE, TRUE)
+  checkEqualsNumeric(base, ttr)
+
+  is.na(base) <- 1:5
+  ttr <- runCov(x, x, 6, "all.obs", TRUE, TRUE)
+  checkEqualsNumeric(base, ttr)
+}
+
 
 # Correlation
 test.runCor <- function() {
@@ -160,6 +229,38 @@ test.runMAD <- function() {
   checkEqualsNumeric( tail(runMAD(input$all$Close,250),1), mad(input$all$Close) )
 }
 
+test.runMAD.cumulative <- function() {
+  cummad <- compiler::cmpfun(
+    function(x) {
+      mad <- x * NA_real_
+      for (i in seq_along(x)) {
+        y <- x[1:i]
+        mad[i] <- mad(y)
+      }
+      mad
+    }
+  )
+  x <- input$all$Close
+  base <- cummad(x)
+  is.na(base) <- 1:4
+  ttr <- runMAD(x, 5, cumulative = TRUE)
+  checkEqualsNumeric(base, ttr)
+
+  is.na(base) <- 1:5
+  ttr <- runMAD(x, 6, cumulative = TRUE)
+  checkEqualsNumeric(base, ttr)
+}
+
+test.runMAD.cumulative.leading.NA <- function() {
+  na <- rep(NA, 10)
+  x <- input$all$Close
+  xmed <- runMAD(x, 1, cumulative = TRUE)
+  y <- c(na, input$all$Close)
+  ymed <- runMAD(y, 1, cumulative = TRUE)
+  checkEqualsNumeric(ymed, c(na, xmed))
+}
+
+
 # Percent Rank
 test.runPercentRank_exact.multiplier_bounds <- function() {
   x <- input$all$Close
@@ -194,19 +295,23 @@ test.runPercentRank_exact.multiplier_eq1 <- function() {
 test.runPercentRank_cumulTRUE_exact.multiplier_eq0 <- function() {
   xrank <- c(0, 0, 2, 0, 4, 1, 0, 2, 3, 3, 0, 8,
              4, 7, 10, 13, 11, 14, 4, 6) / 1:20
+  xrank[1:9] <- NA
+  xrank[10] <- 0
   checkIdentical(xrank, runPercentRank(xdata, 10, TRUE, 0))
 }
 
 test.runPercentRank_cumulTRUE_exact.multiplier_eq0.5 <- function() {
   xrank <- (c(0, 0, 2, 0, 4, 1, 0, 2, 3, 3, 0, 8,
              4, 7, 10, 13, 11, 14, 4, 6) + 0.5) / 1:20
-  #xrank[1] <- 0
+  xrank[1:9] <- NA
+  xrank[10] <- 0.5
   checkIdentical(xrank, runPercentRank(xdata, 10, TRUE, 0.5))
 }
 
 test.runPercentRank_cumulTRUE_exact.multiplier_eq1 <- function() {
   xrank <- (c(0, 0, 2, 0, 4, 1, 0, 2, 3, 3, 0, 8,
              4, 7, 10, 13, 11, 14, 4, 6) + 1) / 1:20
-  #xrank[1] <- 0
+  xrank[1:9] <- NA
+  xrank[10] <- 1
   checkIdentical(xrank, runPercentRank(xdata, 10, TRUE, 1))
 }
